/*
Copyright (C) 2003  Francesco Stablum "DuskDruid"
<duskdruid @ inventati org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "myheaders.h"

/*extern declarations*/
extern int debug_flag;
extern int file_flag;
extern int silent_flag;
/*structure for tracking a connection*/

struct conn_t ** stack; /*all connections, ends on a 0*/
int conn_num; /* number of active connections */

extern int writeheaders_flag;
extern int writestatus_flag;
extern int writeoffset_flag;

int calloc_stack()
{
  stack=(struct conn_t **)S_calloc(4,MAXCONN);
}

int * seek_zero(int * ptr)
{
  int count=0; 
   
  while(*ptr&&(count<MAXCONN))
    {
      ptr++;
      count++;
    }
  if(debug_flag)
    printf("*d* seek_zero(): found free pointer at %x\n",ptr);
  return ptr;  
}
int status_switch(struct conn_t * conn,int status)
{
  set_tracked_color(conn->conn_number);
  if(!silent_flag)
    display_status(stdout,conn,status);
  if(writestatus_flag)
    {
      if(!(conn->file_opened))
	open_file(conn);
      display_status(conn->client_file,conn,status);
      display_status(conn->server_file,conn,status);
    }

	
    
  conn->status=status;
}

int newtrack(struct in_addr client, 
	     struct in_addr server,
	     u_int32_t seq,
	     u_int16_t client_port,
	     u_int16_t server_port)
{
  struct conn_t ** conn;
  conn_num++;
  if(debug_flag)
    printf("*d* newtrack(): new connection(%d):\n"
	   "*d*             seq: %u \n"
	   "*d*             src: %s:%u dst: %s:%u\n",
	   conn_num,
	   seq, 
	   inet_ntoa(client),
	   ntohs(client_port),
	   inet_ntoa(server),
	   ntohs(server_port)
	   );

  /* seek for space in the stack*/
  conn=(struct conn_t **)seek_zero((int *)stack);
  if(debug_flag)
    printf("*d* newtrack(): sizeof(struct conn_t)=%d, conn=%x\n", sizeof(struct conn_t),conn);
  *conn=(struct conn_t *)S_calloc(sizeof(struct conn_t),1);
  if(debug_flag)
    printf("*d*: *conn(%x)\n", *conn);

  (*conn)->client_ip=client.s_addr;
  (*conn)->server_ip=server.s_addr;
  (*conn)->client_port=client_port;
  (*conn)->server_port=server_port;
  (*conn)->client_seq=seq;
  (*conn)->client_data=(char *)S_calloc(16,1);
  (*conn)->server_data=(char *)S_calloc(16,1);
  (*conn)->conn_number=conn_num;
  status_switch(*conn,1);
  

}

/* old file_write(struct conn_t * conn_ptr,
	       u_int32_t data_off, /* offset from beginning 
	       u_char * payload,
	       int payload_len,
	       int part /* client (1) or server (2) ? 
	       )
{ 
  int curr_pos;
  printf("DACANC: file_write(conn_ptr=%x, data_off=%u, payload=%x, payload_len=%d, part=%d)\n"
	 "        conn_ptr->client_file:%x, conn_ptr->server_file:%x\n",
	 conn_ptr,
	 data_off,
	 payload,
	 payload_len,
	 part
	 );
  if(part==1) 
    {
      if(data_off>0)
	if (fseek(conn_ptr->client_file,data_off,SEEK_SET)&&debug_flag)
	  printf("*d* fseek() error\n");
      else
	rewind(conn_ptr->client_file);
      
      for(curr_pos=0;curr_pos<=payload_len;curr_pos++)
	fputc((int)*(payload+curr_pos),conn_ptr->client_file);      
    }

  if(part==2)
    {
      if(data_off>0)
	if (fseek(conn_ptr->server_file,data_off,SEEK_SET)&&debug_flag)
	  printf("*d* fseek() error\n");
      else
	rewind(conn_ptr->server_file);
      
      for(curr_pos=0;curr_pos<=payload_len;curr_pos++)
	fputc((int)*(payload+curr_pos),conn_ptr->server_file);
    }
}
*/
int data_write(struct conn_t * conn_ptr,
	       u_int32_t data_off, /* offset from beginning */
	       u_char * payload,
	       int payload_len,
	       int part /* client (1) or server (2) ? */
	       )
{
  u_char *new_ptr;
  if (payload_len>0)
    {
      if ((data_off+payload_len)<MAXMALLOC)
	{
	  /* allocating a bigger quantity of memory */
	  if(new_ptr=(u_char *)S_calloc(
			       ((payload_len+data_off)/1024)+1,
				2048/* FIXME: with this as 1024 SEGFAULT: why?*/				))
	    {
	      if (part==1)
		/* client data */
		{
		  /* saving old data */
		  memcpy(new_ptr,
			 conn_ptr->client_data,
			 conn_ptr->client_data_size);
		  /* freeing old-allocated memory */
		  S_free(conn_ptr->client_data);
		  conn_ptr->client_data=new_ptr;
		  /* storing new data */
		  memcpy(conn_ptr->client_data+data_off,payload,payload_len);
		  conn_ptr->client_data_size=data_off+payload_len;
		  
		}
	      else
		/* server data */
		{
		  /* saving old data */
		  memcpy(new_ptr,
			 conn_ptr->server_data,
			 conn_ptr->server_data_size);
		  if(debug_flag)
		    printf("*d* The next instruction sometimes goes in segfault!\n");
		  S_free(conn_ptr->server_data); /* FIXME - BUG sometimes SEGFAULT */
		  conn_ptr->server_data=new_ptr;
		  /* storing new data */
		  memcpy(conn_ptr->server_data+data_off,payload,payload_len);
		  conn_ptr->server_data_size=data_off+payload_len;
		}
	      /* writing to file */
	    }
	  else
	    ;//FIXME:memory not allocated!!!
	  if(file_flag)
	    {
	      if(conn_ptr->file_opened==0)
		open_file(conn_ptr); 
	      if(!writeheaders_flag
		 &&
		 !writestatus_flag
		 &&
		 !writeoffset_flag)
		/* if write header/status is NOT selected */
		{
		  file_write(conn_ptr,part); /* write the whole data */
		}
	      else	      
		/* if write header/status is selected */
		{
		  /* 
		     here should be nothing because not 
		     all packets go to data_write functions, 
		     but only the recognized ones 
		  */
		}
		
	    }
	}
      else
	printf("*E* WARNING: VERY BAD ERROR: \n"
	       "*E* data_off: %u payload_len: %u\n",
	       data_off,
	       payload_len);
    }
}
