/*
Copyright (C) 2003, 2004  Francesco Stablum "DuskDruid"
<duskdruid @ inventati org>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "myheaders.h"

extern struct conn_t ** stack;
extern int debug_flag;

int verify(struct ip *ippacket,
	   struct tcphdr *tcppacket,
	   u_char *payload,
	   int payload_len
	   )
	
{
 struct conn_t ** stack_ptr;
 struct in_addr in_addr_client;
 struct in_addr in_addr_server;
 int count=0;
 int loop_finished=0;
 u_int32_t data_off=0;
 stack_ptr=stack;

 /*SYN-SENT: (outside of the loop because the loop *need* it)*/	 
 if(match(*stack_ptr,ippacket,tcppacket,
	  -1,-1,
	  0,0,0,0,1,0)
    )
   if (tcppacket->ack_seq!=0||
       tcppacket->seq==0) 
     {
       if (debug_flag)
	 printf("*d* WARNING: strange syn packet: ack_seq: %u seq:%u\n",tcppacket->ack_seq,tcppacket->seq);
     }
   else
     /* IT IS SYN-SENT */
     newtrack(ippacket->ip_src,
	      ippacket->ip_dst, 
	      tcppacket->seq,
	      tcppacket->source,
	      tcppacket->dest
	      ); /* track new connection */



 while(count<MAXCONN&&loop_finished==0)
   /* seek for a connection that matches*/
   {
     count++;
     if (*stack_ptr!=NULL)
       {
	 in_addr_client.s_addr=(*stack_ptr)->client_ip;
	 in_addr_server.s_addr=(*stack_ptr)->server_ip;
	 
	 /* main cycle */
	 /* writing headers if writeheaders_flag to the file */
	 if(match(*stack_ptr,ippacket,tcppacket,
		  3,-1,
		  -1,-1,-1,-1,-1,-1)
	    )
	   {
	     write_banners_and_data(*stack_ptr,ippacket,tcppacket,payload,payload_len);		   }
	 
	 
	 /* SYN-RECEIVED:*/
	 if(match(*stack_ptr,ippacket,tcppacket,
		  2,1,
		  0,1,0,0,1,0)
	    )
	   {  
	     if (tcppacket->ack_seq==0||
		 tcppacket->seq==0) 
	       {
		 if (debug_flag)
		   printf("*d* WARNING: strange syn/ack packet: ack_seq: %u seq:%u\n",
			  tcppacket->ack_seq,
			  tcppacket->seq);
	       }
	       else
		 /* IT IS SYN-RECEIVED */
		 status_switch(*stack_ptr,2);
	     (*stack_ptr)->server_seq=tcppacket->seq;
	     loop_finished=1;
	     
	     if (debug_flag)
	       printf("*d* [stack] found matching connection (%x) in SYN_SENT status:\n"
		      "*d* [stack] client: %s:%u server: %s:%u \n",
		      stack_ptr,
		      inet_ntoa(in_addr_client),
		      ntohs((*stack_ptr)->client_port),
		      inet_ntoa(in_addr_server),
		      ntohs((*stack_ptr)->server_port)
		      );
	   }
	 
	 
	 
	 
	 /* ESTABLISHING... (3rd packet)*/
	 if(match(*stack_ptr,ippacket,tcppacket,
		  1,2,
		  0,1,0,0,0,0)
	    )
	   
	   if (tcppacket->ack_seq==0||
	       tcppacket->seq==0) 
	     {
	       if (debug_flag)
		 printf("*d* WARNING: strange ack packet: ack_seq: %u seq:&u\n",
			tcppacket->ack_seq,
			tcppacket->seq);
	     }
	     else
	       if(ntohl((*stack_ptr)->client_seq)+1==ntohl(tcppacket->seq)&&
		  ntohl((*stack_ptr)->server_seq)+1==ntohl(tcppacket->ack_seq)
		  )
		 {
	   /*this is really the third packet of the 3wh
	     3wh completed!*/
		   if (debug_flag)
		     printf("*d* [stack] found matching connection (%x) in SYN_RECEIVED status:\n"
			    "*d* [stack] client: %s:%u server: %s:%u \n"
			    "*d* [stack] status switched to ESTABLISHED\n",
			    stack_ptr,
			    inet_ntoa(in_addr_client),
			    ntohs((*stack_ptr)->client_port),
			    inet_ntoa(in_addr_server),
			    ntohs((*stack_ptr)->server_port)
			    );			     
		   /*switch connection status to "ESTABLISHED"*/
		   status_switch(*stack_ptr,3);
		   loop_finished=1;	    
		 }
	       else
		 /*strange seq and ack_seq fields */
		 if(debug_flag)
		   printf("*d* WARNING: seq and ack tells this is not the 3rd packet\n"
			  "*d*          of three-way handshake!\n");
	 
	   
	 
	 
	 
	 /* ESTABLISHED: client sends data */
	 if(match(*stack_ptr,ippacket,tcppacket,
		  1,3,
		  0,1,-1,0,0,0)
	    )
	   {
	     data_off=(ntohl(tcppacket->seq)-ntohl((*stack_ptr)->client_seq))-1;
	     if(debug_flag)
	       printf("*d* data_off:%u\n",data_off);
	     data_write(*stack_ptr,
			data_off,
		    payload,
		    payload_len,
		    2
		    ); /* write the data to terminal, memory or file */
	     loop_finished=1;
	   }
       }
     
     
     
     /* ESTABLISHED: server sends data */
     if(match(*stack_ptr,ippacket,tcppacket,
	      2,3,
	      0,1,-1,0,0,0)
	)
       {
	 data_off=(ntohl(tcppacket->seq)-ntohl((*stack_ptr)->server_seq))-1;
	 if(debug_flag)
	   printf("*d* data_off:%u\n",data_off);
	 data_write(*stack_ptr,
		    data_off,
		    payload,
		    payload_len,
		    1
		    ); /* write the data to terminal, memory or file */
	 loop_finished=1;
       }
     
     
     
     
     /* first FIN sent from client*/ 
     if(match(*stack_ptr,ippacket,tcppacket,
	      1,3,
	      0,-1,-1,0,0,1)
	)
       {
	 /* this is a FIN-WAIT-1 status! */
	 status_switch(*stack_ptr,4);
	 (*stack_ptr)->closer=1; /* is the client that closes the connection */
	 loop_finished=1;
       }
        
     
     
     /* first FIN sent from server*/ 
     if(match(*stack_ptr,ippacket,tcppacket,
	      2,3,
	      0,-1,-1,0,0,1)
	)
       {
	 /* this is a FIN-WAIT-1 status! */
	 status_switch(*stack_ptr,4);
	 (*stack_ptr)->closer=2; /* is the server that closes the connection */
	 loop_finished=1;
       }



     /* ACK of first FIN sent from client*/ 
     if(match(*stack_ptr,ippacket,tcppacket,
	      1,4,
	      0,1,-1,0,0,0)
	)
       {
	 /* this is a FIN-WAIT-2/CLOSE-WAIT status! */
	 if((*stack_ptr)->closer==2) 
	   /* is the server that closes the connection */
	   {
	     status_switch(*stack_ptr,5);
	     loop_finished=1;
	   }
       }
          


     /* ACK of first FIN sent from server*/ 
     if(match(*stack_ptr,ippacket,tcppacket,
	      2,4,
	      0,1,-1,0,0,0)
	)
       {
	 /* this is a FIN-WAIT-2/CLOSE-WAIT status! */
	 if((*stack_ptr)->closer==1)
	   /* is the client that closes the connection */
	   {
	     status_switch(*stack_ptr,5);
	     loop_finished=1;
	   }
       }




     /* second FIN sent from client */ 
     if(match(*stack_ptr,ippacket,tcppacket,
	      1,4,
	      0,1,-1,0,0,1)
	||
	match(*stack_ptr,ippacket,tcppacket,
	      1,5,
	      0,1,-1,0,0,1)
	)
       {
	 /* this is a TIME-WAIT/LAST-ACK status! */
	 if((*stack_ptr)->closer==2) 
	   /* is the server that closes the connection */
	   {
	     status_switch(*stack_ptr,6);
	     loop_finished=1;
	   }
       }
          


     /* second FIN sent from server*/ 
     if(match(*stack_ptr,ippacket,tcppacket,
	      2,4,
	      0,1,-1,0,0,1)
	||
	match(*stack_ptr,ippacket,tcppacket,
	      2,5,
	      0,1,-1,0,0,1)
	)
       {
	 /* this is a TIME-WAIT/LAST-ACK status! */
	 if((*stack_ptr)->closer==1)
	   /* is the client that closes the connection */
	   {
	     status_switch(*stack_ptr,6);
	     loop_finished=1;
	   }
       }



     /* LAST-ACK sent from client */
     if(match(*stack_ptr,ippacket,tcppacket,
	      1,6,
	      0,1,-1,0,0,0)
	)
     {
       if((*stack_ptr)->closer==1)
	   /* is the server that closes the connection */
	   /* this is a LAST-ACK status! */
	   {
	     status_switch(*stack_ptr,7);
	     loop_finished=1;
	   }
     }

     /* LAST-ACK sent from server */
     if(match(*stack_ptr,ippacket,tcppacket,
	      2,6,
	      0,1,-1,0,0,0)
	)
     {
       if((*stack_ptr)->closer==2)
	   /* is the client that closes the connection */
	   /* this is a LAST-ACK status! */
	   {
	     status_switch(*stack_ptr,7);
	     loop_finished=1;
	   }
     }
     stack_ptr++;
   }
 
}

